/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web;

import java.net.URL;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.StaleObjectStateException;
import org.hibernate.Transaction;
import net.smartlab.config.Configuration;
import net.smartlab.config.ConfigurationException;
import net.smartlab.config.XMLConfiguration;

/**
 * @author rlogiacco
 */
public abstract class Domain implements ManageableDomain {

	/**
	 * Provides logging capabilities to the domain.
	 */
	protected final Log logger = LogFactory.getLog(this.getClass());

	/**
	 * The internal configuration.
	 */
	protected Configuration config;


	/**
	 * Starts a new transaction, allowing all the operations performed in the
	 * same context to be atomically applied (commit) or reverted (rollback).
	 * The context is identifyied by a <code>BusinessObjectFactory</code>
	 * instance but the context spans over all the instances sharing the same
	 * configuration.
	 * 
	 * @param factory the identifier for the context
	 * @return an object representation of the transaction.
	 * @throws BusinessException if something wrong occurs during the operation.
	 */
	protected Transaction begin(BusinessObjectFactory factory) throws BusinessException {
		try {
			return factory.current().beginTransaction();
		} catch (HibernateException he) {
			logger.error("[ smartweb ] failed to begin transaction");
			throw new BusinessException("persistance.error.begin", he);
		} catch (DAOException boe) {
			throw new BusinessException(boe);
		}
	}

	/**
	 * Commits the specified transaction applying all the changes happened since
	 * its beginning.
	 * 
	 * @param transaction the transaction to commit.
	 * @throws BusinessException if something wrong occurs during the operation.
	 */
	protected void commit(Transaction transaction) throws BusinessException {
		try {
			transaction.commit();
		} catch (StaleObjectStateException sose) {
			logger.info("[ smartweb ] optimistical locking collision");
			throw new BusinessException("persistance.locking.collision", sose);
		} catch (HibernateException he) {
			logger.error("[ smartweb ] failed to commit transaction");
			throw new BusinessException("persistance.error.commit", he);
		}
	}

	/**
	 * Rollbacks the specified transaction reverting all the changes happened
	 * since its beginning.
	 * 
	 * @param transaction the transaction to rollback.
	 * @throws BusinessException if something wrong occurs during the operation.
	 */
	protected void rollback(Transaction transaction) throws BusinessException {
		try {
			transaction.rollback();
		} catch (HibernateException he) {
			logger.warn("[ smartweb ] failed to rollback transaction");
		}
	}

	/**
	 * Subclasses should make their own constructors private and behave like
	 * singletons.
	 */
	protected Domain() {
		if (logger.isDebugEnabled()) {
			logger.debug(this.getClass().getName() + " instantiated.");
		}
		String archive = Domain.getLastArchiveName(this.getClass());
		URL file = Domain.getResource(this.getClass(), new String[] {"/META-INF" + archive + ".xml",
				"/META-INF/smartweb.jar.xml"});
		try {
			this.config = new XMLConfiguration(file);
		} catch (ConfigurationException ce) {
			throw new RuntimeException("Failed to configure domain", ce);
		}
	}

	/**
	 * Returns the <code>Configuration</code> instance used by this instance.
	 * 
	 * @return the <code>Configuration</code> instance used by this instance.
	 */
	public Configuration getConfiguration() {
		return config;
	}

	/**
	 * Returns the first resource in the list available in the specified
	 * context.
	 * 
	 * @param context the class identifying the search context.
	 * @param names the list, ordered from first to last, of resources to search
	 *        for.
	 * @return the first resource in the list available in the context.
	 */
	public static URL getResource(Class context, String[] names) {
		Log logger = LogFactory.getLog(context);
		URL resource = null;
		for (int i = 0; i < names.length; i++) {
			resource = context.getResource(names[i]);
			logger.trace("   trying `" + names[i] + "`");
			if (resource != null) {
				break;
			}
		}
		logger.debug("   resource is `" + resource + "`");
		return resource;
	}

	/**
	 * Returns the name of the last archive containing the class or an
	 * <code>empty</code> string if the class is in no archive.
	 * 
	 * @param type the class to search for.
	 * @return the name of the last archive containing the class or an
	 *         <code>empty</code> string if the class is in no archive.
	 */
	public static String getLastArchiveName(Class type) {
		Log logger = LogFactory.getLog(type);
		String path = type.getProtectionDomain().getCodeSource().getLocation().getFile();
		logger.debug("   archive path is `" + path + "`");
		return path.substring(path.lastIndexOf('/') + 1);
	}
}